@@ -4,7 +4,7 @@
 # also regenerates all aclocal.m4, config.h.in, Makefile.in, configure files
 # with new versions of autoconf or automake.
 #
-# This script requires autoconf-2.64..2.72 and automake-1.13..1.17 in the PATH.
+# This script requires autoconf-2.64..2.72 and automake-1.13..1.18 in the PATH.
 
 # Copyright (C) 2003-2025 Free Software Foundation, Inc.
 #
@@ -21,9 +21,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-# Prerequisite (if not used from a released tarball): either
-#   - the GNULIB_SRCDIR environment variable pointing to a gnulib checkout, or
-#   - a preceding invocation of './autopull.sh'.
+# Prerequisite (if not used from a released tarball):
+#   - A preceding invocation of './autopull.sh'.
+#   - If the GNULIB_SRCDIR environment variable was set during the
+#     './autopull.sh' invocation, pointing to a gnulib checkout, it should
+#     still be set now, during the './autogen.sh' invocation.
 #
 # Usage: ./autogen.sh [--skip-gnulib]
 #
@@ -99,33 +101,48 @@ if ! $skip_gnulib; then
     basename-lgpl
     binary-io
     bool
+    c-ctype
+    c-strtold
     closeout
     error
-    getopt-gnu
+    fzprintf-posix
     gettext-h
     havelib
+    mbrtoc32
+    mbszero
     memmove
     noreturn
+    options
     progname
     propername
+    quote
     relocatable-prog
     setlocale
     sigpipe
+    stdint-h
     stdio-h
     stdlib-h
+    strtoimax
+    strtold
     strtoul
+    strtoumax
     unistd-h
     unlocked-io
     xalloc
     xstring-buffer
+    xstrtold
   '
   GNULIB_MODULES_RUNTIME_OTHER='
     gettext-runtime-misc
     ansi-c++-opt
     csharpcomp-script
+    d
+    dcomp-script
     java
     javacomp-script
     manywarnings
+    modula2
+    modula2comp-script
   '
   $GNULIB_TOOL --dir=gettext-runtime --lib=libgrt --source-base=gnulib-lib --m4-base=gnulib-m4 --no-libtool --local-dir=gnulib-local --local-symlink \
     --import $GNULIB_MODULES_RUNTIME_FOR_SRC $GNULIB_MODULES_RUNTIME_OTHER || exit $?
@@ -178,6 +195,7 @@ if ! $skip_gnulib; then
     c-strcase
     c-strcasestr
     c-strstr
+    carray-list
     clean-temp
     closedir
     closeout
@@ -200,9 +218,12 @@ if ! $skip_gnulib; then
     gcd
     getaddrinfo
     getline
-    getopt-gnu
     getrusage
     gettext-h
+    gocomp-script
+    hash-map
+    hash-set
+    hashkey-string
     iconv
     javacomp
     javaexec
@@ -225,6 +246,7 @@ if ! $skip_gnulib; then
     open
     opendir
     openmp-init
+    options
     pipe-filter-ii
     progname
     propername
@@ -232,6 +254,7 @@ if ! $skip_gnulib; then
     readdir
     relocatable-prog
     relocatable-script
+    set
     setlocale
     sf-istream
     sh-filename
@@ -272,7 +295,11 @@ if ! $skip_gnulib; then
     unistr/u8-mbtouc
     unistr/u8-mbtoucr
     unistr/u8-uctomb
+    unistr/u16-check
+    unistr/u16-to-u8
     unistr/u16-mbtouc
+    unistr/u32-check
+    unistr/u32-to-u8
     uniwidth/width
     unlocked-io
     unsetenv
@@ -283,56 +310,148 @@ if ! $skip_gnulib; then
     xalloc
     xconcat-filename
     xerror
+    xlist
     xmalloca
+    xmap
     xmemdup0
+    xset
     xsetenv
     xstrerror
     xstriconv
     xstriconveh
     xstring-buffer
+    xstring-buffer-reversed
     xstring-desc
     xvasprintf
   '
   # Common dependencies of GNULIB_MODULES_TOOLS_FOR_SRC and GNULIB_MODULES_TOOLS_FOR_LIBGREP.
   GNULIB_MODULES_TOOLS_FOR_SRC_COMMON_DEPENDENCIES='
+    absolute-header
+    alignasof
     alloca-opt
+    assert-h
+    attribute
+    basename-lgpl
     bool
+    btowc
+    builtin-expect
+    c99
+    calloc-gnu
+    calloc-posix
+    cloexec
+    close
+    double-slash-root
+    dup2
+    errno-h
+    error
+    error-h
+    exitfail
     extensions
+    extensions-aix
+    extern-inline
+    fcntl
+    fcntl-h
+    fd-hook
+    filename
+    flexmember
+    fstat
+    gen-header
+    getdtablesize
+    getprogname
     gettext-h
+    gnulib-i18n
+    hard-locale
+    ialloc
+    idx
     include_next
-    locale-h
+    intprops
+    inttypes-h-incomplete
+    iswblank
+    iswctype
+    iswdigit
+    iswpunct
+    iswxdigit
+    largefile
+    libc-config
+    limits-h
     localcharset
+    localeconv
+    locale-h
+    lock
+    lstat
+    malloca
+    malloc-gnu
     malloc-posix
     mbrtowc
     mbsinit
+    mbszero
+    mbtowc
+    memchr
+    memcmp
+    memmove
+    minmax
+    msvc-inval
+    msvc-nothrow
     multiarch
+    obstack
+    once
+    open
+    pathmax
+    pthread-h
+    pthread-once
+    reallocarray
+    realloc-posix
+    sched-h
     setlocale-null
+    setlocale-null-unlocked
     snippet/arg-nonnull
     snippet/c++defs
+    snippet/_Noreturn
     snippet/warn-on-use
     ssize_t
+    stat
+    stat-time
+    stdckdint-h
     stddef-h
+    std-gnu11
     stdint-h
+    stdio-h
     stdlib-h
     streq
+    strerror
+    strerror-override
+    string-h
+    sys_stat-h
+    sys_types-h
+    threadlib
+    time-h
     unistd-h
+    vararrays
     verify
     wchar-h
+    wctype
     wctype-h
     windows-mutex
     windows-once
     windows-recmutex
     windows-rwlock
+    xalloc
+    xalloc-die
+    xalloc-oversized
   '
   GNULIB_MODULES_TOOLS_OTHER='
     gettext-tools-misc
     ansi-c++-opt
     csharpcomp-script
     csharpexec-script
+    d
+    dcomp-script
     java
     javacomp-script
     javaexec-script
     manywarnings
+    modula2
+    modula2comp-script
     stdint-h
     test-xfail
   '
@@ -349,13 +468,14 @@ if ! $skip_gnulib; then
     --avoid=float-h-tests \
     --avoid=hashcode-string1 \
     --avoid=fdutimensat-tests --avoid=futimens-tests --avoid=utime-tests --avoid=utimens-tests --avoid=utimensat-tests \
-    --avoid=array-list-tests --avoid=array-map-tests --avoid=hash-map-tests --avoid=linked-list-tests --avoid=linkedhash-list-tests \
+    --avoid=array-list-tests --avoid=array-map-tests --avoid=array-oset-tests --avoid=carray-list-tests --avoid=linked-list-tests --avoid=linkedhash-list-tests \
     --avoid=uninorm/decomposing-form-tests \
     `for m in $GNULIB_MODULES_TOOLS_LIBUNISTRING_TESTS; do echo --avoid=$m; done` \
     $GNULIB_MODULES_LIBGETTEXTLIB || exit $?
   $GNULIB_TOOL --copy-file m4/libtextstyle.m4 gettext-tools/gnulib-m4/libtextstyle.m4 || exit $?
   # In gettext-tools/libgrep:
   GNULIB_MODULES_TOOLS_FOR_LIBGREP='
+    kwset
     mbrlen
     regex
   '
