@@ -1,6 +1,6 @@
 /* Analyze differences between two vectors.
 
-   Copyright (C) 1988-1989, 1992-1995, 2001-2004, 2006-2024 Free Software
+   Copyright (C) 1988-1989, 1992-1995, 2001-2004, 2006-2025 Free Software
    Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -77,6 +77,11 @@
      #include "minmax.h"
  */
 
+/* This file uses _GL_GNUC_PREREQ.  */
+#if !_GL_CONFIG_H_INCLUDED
+ #error "Please include config.h first."
+#endif
+
 /* Maximum value of type OFFSET.  */
 #ifndef OFFSET_MAX
 # define OFFSET_MAX \
@@ -93,7 +98,7 @@
 #endif
 
 /* Suppress gcc's "...may be used before initialized" warnings,
-   generated by GCC versions up to at least GCC 14.2.
+   generated by GCC versions up to at least GCC 15.1.
    Likewise for gcc -fanalyzer's "use of uninitialized value" warnings.  */
 #if _GL_GNUC_PREREQ (4, 7)
 # pragma GCC diagnostic push
