@@ -1,10 +1,10 @@
 /* Charset conversion with out-of-memory checking.
-   Copyright (C) 2001-2007, 2009-2022 Free Software Foundation, Inc.
+   Copyright (C) 2001-2007, 2009-2025 Free Software Foundation, Inc.
    Written by Bruno Haible and Simon Josefsson.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -18,6 +18,11 @@
 #ifndef _XSTRICONVEH_H
 #define _XSTRICONVEH_H
 
+/* This file uses _GL_ATTRIBUTE_MALLOC, HAVE_ICONV.  */
+#if !_GL_CONFIG_H_INCLUDED
+ #error "Please include config.h first."
+#endif
+
 #include <stdlib.h>
 
 /* Get the 'enum iconv_ilseq_handler' and iconveh_t types, and the
