@@ -1,10 +1,10 @@
 /*
- * Copyright (C) 2010-2022 Free Software Foundation, Inc.
+ * Copyright (C) 2010-2025 Free Software Foundation, Inc.
  * Written by Eric Blake
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -64,5 +64,5 @@ main (void)
       while (j--);
     }
 
-  return 0;
+  return test_exit_status;
 }
