@@ -20,7 +20,6 @@
 # include "config.h"
 #endif
 
-#include <getopt.h>
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,6 +28,7 @@
 #include <textstyle.h>
 
 #include <error.h>
+#include "options.h"
 #include "noreturn.h"
 #include "closeout.h"
 #include "dir-list.h"
@@ -60,34 +60,6 @@
 /* Force output of PO file even if empty.  */
 static int force_po;
 
-/* Long options.  */
-static const struct option long_options[] =
-{
-  { "add-location", optional_argument, NULL, 'n' },
-  { "color", optional_argument, NULL, CHAR_MAX + 5 },
-  { "directory", required_argument, NULL, 'D' },
-  { "escape", no_argument, NULL, 'E' },
-  { "force-po", no_argument, &force_po, 1 },
-  { "help", no_argument, NULL, 'h' },
-  { "indent", no_argument, NULL, 'i' },
-  { "lang", required_argument, NULL, CHAR_MAX + 4 },
-  { "no-escape", no_argument, NULL, 'e' },
-  { "no-location", no_argument, NULL, CHAR_MAX + 7 },
-  { "no-wrap", no_argument, NULL, CHAR_MAX + 1 },
-  { "output-file", required_argument, NULL, 'o' },
-  { "properties-input", no_argument, NULL, 'P' },
-  { "properties-output", no_argument, NULL, 'p' },
-  { "sort-by-file", no_argument, NULL, 'F' },
-  { "sort-output", no_argument, NULL, 's' },
-  { "strict", no_argument, NULL, 'S' },
-  { "stringtable-input", no_argument, NULL, CHAR_MAX + 2 },
-  { "stringtable-output", no_argument, NULL, CHAR_MAX + 3 },
-  { "style", required_argument, NULL, CHAR_MAX + 6 },
-  { "version", no_argument, NULL, 'V' },
-  { "width", required_argument, NULL, 'w' },
-  { NULL, 0, NULL, 0 }
-};
-
 
 /* Forward declaration of local functions.  */
 _GL_NORETURN_FUNC static void usage (int status);
@@ -97,7 +69,6 @@ static msgdomain_list_ty *fill_header (msgdomain_list_ty *mdlp);
 int
 main (int argc, char **argv)
 {
-  int opt;
   bool do_help;
   bool do_version;
   char *output_file;
@@ -131,12 +102,41 @@ main (int argc, char **argv)
   do_version = false;
   output_file = NULL;
 
-  while ((opt = getopt_long (argc, argv,
-                             "D:eEFhin:o:pPsVw:",
-                             long_options, NULL)) != EOF)
+  /* Parse command line options.  */
+  BEGIN_ALLOW_OMITTING_FIELD_INITIALIZERS
+  static const struct program_option options[] =
+  {
+    { "add-location",       CHAR_MAX + 'n', optional_argument },
+    { NULL,                 'n',            no_argument       },
+    { "color",              CHAR_MAX + 5,   optional_argument },
+    { "directory",          'D',            required_argument },
+    { "escape",             'E',            no_argument       },
+    { "force-po",           0,              no_argument,      &force_po, 1 },
+    { "help",               'h',            no_argument       },
+    { "indent",             'i',            no_argument       },
+    { "lang",               CHAR_MAX + 4,   required_argument },
+    { "no-escape",          'e',            no_argument       },
+    { "no-location",        CHAR_MAX + 7,   no_argument       },
+    { "no-wrap",            CHAR_MAX + 1,   no_argument       },
+    { "output-file",        'o',            required_argument },
+    { "properties-input",   'P',            no_argument       },
+    { "properties-output",  'p',            no_argument       },
+    { "sort-by-file",       'F',            no_argument       },
+    { "sort-output",        's',            no_argument       },
+    { "strict",             CHAR_MAX + 8,   no_argument       },
+    { "stringtable-input",  CHAR_MAX + 2,   no_argument       },
+    { "stringtable-output", CHAR_MAX + 3,   no_argument       },
+    { "style",              CHAR_MAX + 6,   required_argument },
+    { "version",            'V',            no_argument       },
+    { "width",              'w',            required_argument },
+  };
+  END_ALLOW_OMITTING_FIELD_INITIALIZERS
+  start_options (argc, argv, options, MOVE_OPTIONS_FIRST, 0);
+  int opt;
+  while ((opt = get_next_option ()) != -1)
     switch (opt)
       {
-      case '\0':                /* Long option.  */
+      case '\0':                /* Long option with key == 0.  */
         break;
 
       case 'D':
@@ -163,7 +163,8 @@ main (int argc, char **argv)
         message_print_style_indent ();
         break;
 
-      case 'n':
+      case 'n':            /* -n */
+      case CHAR_MAX + 'n': /* --add-location[={full|yes|file|never|no}] */
         if (handle_filepos_comment_option (optarg))
           usage (EXIT_FAILURE);
         break;
@@ -184,7 +185,7 @@ main (int argc, char **argv)
         sort_by_msgid = true;
         break;
 
-      case 'S':
+      case CHAR_MAX + 8: /* --strict */
         message_print_style_uniforum ();
         break;
 
@@ -283,16 +284,16 @@ There is NO WARRANTY, to the extent permitted by law.\n\
   /* Add English translations.  */
   result = msgdomain_list_english (result);
 
+  /* Set the Language field in the header.  */
+  if (catalogname != NULL)
+    msgdomain_list_set_header_field (result, "Language:", catalogname);
+
   /* Sort the results.  */
   if (sort_by_filepos)
     msgdomain_list_sort_by_filepos (result);
   else if (sort_by_msgid)
     msgdomain_list_sort_by_msgid (result);
 
-  /* Set the Language field in the header.  */
-  if (catalogname != NULL)
-    msgdomain_list_set_header_field (result, "Language:", catalogname);
-
   /* Write the merged message list out.  */
   msgdomain_list_print (result, output_file, output_syntax,
                         textmode_xerror_handler, force_po, false);
