@@ -1,5 +1,5 @@
 /* Initializes a new PO file.
-   Copyright (C) 2001-2024 Free Software Foundation, Inc.
+   Copyright (C) 2001-2025 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -24,7 +24,6 @@
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <getopt.h>
 #include <limits.h>
 #include <locale.h>
 #include <stdint.h>
@@ -42,7 +41,7 @@
 #include <textstyle.h>
 
 #include <error.h>
-
+#include "options.h"
 #include "noreturn.h"
 #include "closeout.h"
 #include "error-progname.h"
@@ -78,6 +77,7 @@
 #include "spawn-pipe.h"
 #include "wait-process.h"
 #include "xsetenv.h"
+#include "xstriconv.h"
 #include "str-list.h"
 #include "propername.h"
 #include "gettext.h"
@@ -106,26 +106,6 @@ static const char *language;
 /* If true, the user is not considered to be the translator.  */
 static bool no_translator;
 
-/* Long options.  */
-static const struct option long_options[] =
-{
-  { "color", optional_argument, NULL, CHAR_MAX + 5 },
-  { "help", no_argument, NULL, 'h' },
-  { "input", required_argument, NULL, 'i' },
-  { "locale", required_argument, NULL, 'l' },
-  { "no-translator", no_argument, NULL, CHAR_MAX + 1 },
-  { "no-wrap", no_argument, NULL, CHAR_MAX + 2 },
-  { "output-file", required_argument, NULL, 'o' },
-  { "properties-input", no_argument, NULL, 'P' },
-  { "properties-output", no_argument, NULL, 'p' },
-  { "stringtable-input", no_argument, NULL, CHAR_MAX + 3 },
-  { "stringtable-output", no_argument, NULL, CHAR_MAX + 4 },
-  { "style", required_argument, NULL, CHAR_MAX + 6 },
-  { "version", no_argument, NULL, 'V' },
-  { "width", required_argument, NULL, 'w' },
-  { NULL, 0, NULL, 0 }
-};
-
 /* Forward declaration of local functions.  */
 _GL_NORETURN_FUNC static void usage (int status);
 static const char *find_pot (void);
@@ -139,7 +119,6 @@ static msgdomain_list_ty *update_msgstr_plurals (msgdomain_list_ty *mdlp);
 int
 main (int argc, char **argv)
 {
-  int opt;
   bool do_help;
   bool do_version;
   char *output_file;
@@ -172,11 +151,32 @@ main (int argc, char **argv)
   input_file = NULL;
   locale = NULL;
 
-  while ((opt = getopt_long (argc, argv, "hi:l:o:pPVw:", long_options, NULL))
-         != EOF)
+  /* Parse command line options.  */
+  BEGIN_ALLOW_OMITTING_FIELD_INITIALIZERS
+  static const struct program_option options[] =
+  {
+    { "color",              CHAR_MAX + 5, optional_argument },
+    { "help",               'h',          no_argument       },
+    { "input",              'i',          required_argument },
+    { "locale",             'l',          required_argument },
+    { "no-translator",      CHAR_MAX + 1, no_argument       },
+    { "no-wrap",            CHAR_MAX + 2, no_argument       },
+    { "output-file",        'o',          required_argument },
+    { "properties-input",   'P',          no_argument       },
+    { "properties-output",  'p',          no_argument       },
+    { "stringtable-input",  CHAR_MAX + 3, no_argument       },
+    { "stringtable-output", CHAR_MAX + 4, no_argument       },
+    { "style",              CHAR_MAX + 6, required_argument },
+    { "version",            'V',          no_argument       },
+    { "width",              'w',          required_argument },
+  };
+  END_ALLOW_OMITTING_FIELD_INITIALIZERS
+  start_options (argc, argv, options, MOVE_OPTIONS_FIRST, 0);
+  int opt;
+  while ((opt = get_next_option ()) != -1)
     switch (opt)
       {
-      case '\0':                /* Long option.  */
+      case '\0':                /* Long option with key == 0.  */
         break;
 
       case 'h':
@@ -263,7 +263,7 @@ License GPLv3+: GNU GPL version 3 or later <%s>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "2001-2024", "https://gnu.org/licenses/gpl.html");
+              "2001-2025", "https://gnu.org/licenses/gpl.html");
       printf (_("Written by %s.\n"), proper_name ("Bruno Haible"));
       exit (EXIT_SUCCESS);
     }
@@ -568,6 +568,7 @@ catalogname_for_locale (const char *locale)
     "fy_NL",    /* Western Frisian      Netherlands */
     "ga_IE",    /* Irish        Ireland */
     "gd_GB",    /* Scottish Gaelic      Britain */
+    "gl_ES",    /* Galician     Spain */
     "gon_IN",   /* Gondi        India */
     "gsw_CH",   /* Swiss German Switzerland */
     "gu_IN",    /* Gujarati     India */
@@ -685,6 +686,7 @@ catalogname_for_locale (const char *locale)
     "suk_TZ",   /* Sukuma       Tanzania */
     "sus_GN",   /* Susu         Guinea */
     "sv_SE",    /* Swedish      Sweden */
+    "ta_IN",    /* Tamil        India */
     "te_IN",    /* Telugu       India */
     "tem_SL",   /* Timne        Sierra Leone */
     "tet_ID",   /* Tetum        Indonesia */
@@ -779,6 +781,8 @@ language_of_locale (const char *locale)
 }
 
 
+/* ---------------------- fill_header and subroutines ---------------------- */
+
 /* Return the most likely desired charset for the PO file, as a portable
    charset name.  */
 static const char *
@@ -820,6 +824,11 @@ canonical_locale_charset ()
 }
 
 
+/* The desired charset for the PO file.
+   Determined at the beginning of fill_header().  */
+static const char *output_charset;
+
+
 /* Return the English name of the language.  */
 static const char *
 englishname_of_language ()
@@ -1353,27 +1362,7 @@ mime_version ()
 static const char *
 content_type (const char *header)
 {
-  bool was_utf8;
-  const char *old_field;
-
-  /* If the POT file contains charset=UTF-8, it means that the POT file
-     contains non-ASCII characters, and we keep the UTF-8 encoding.
-     Otherwise, when the POT file is plain ASCII, we use the locale's
-     encoding.  */
-  was_utf8 = false;
-  old_field = get_field (header, "Content-Type");
-  if (old_field != NULL)
-    {
-      const char *charsetstr = c_strstr (old_field, "charset=");
-
-      if (charsetstr != NULL)
-        {
-          charsetstr += strlen ("charset=");
-          was_utf8 = (c_strcasecmp (charsetstr, "UTF-8") == 0);
-        }
-    }
-  return xasprintf ("text/plain; charset=%s",
-                    was_utf8 ? "UTF-8" : canonical_locale_charset ());
+  return xasprintf ("text/plain; charset=%s", output_charset);
 }
 
 
@@ -1643,25 +1632,21 @@ get_title ()
   /* This is tricky.  We want the translation in the given locale specified by
      the command line, not the current locale.  But we want it in the encoding
      that we put into the header entry, not the encoding of that locale.
-     We could avoid the use of OUTPUT_CHARSET by using a separate message
-     catalog and bind_textdomain_codeset(), but that doesn't seem worth the
-     trouble for one single message.  */
-  const char *encoding;
+     We could avoid the use of xstr_iconv() by using a separate message catalog
+     and bind_textdomain_codeset(), but that doesn't seem worth the trouble
+     for one single message.  */
   const char *tmp;
   char *old_LC_ALL;
   char *old_LANGUAGE;
-  char *old_OUTPUT_CHARSET;
   const char *msgid;
   const char *english;
   const char *result;
 
-  encoding = canonical_locale_charset ();
-
   /* First, the English title.  */
   english = xasprintf ("%s translations for %%s package",
                        englishname_of_language ());
 
-  /* Save LC_ALL, LANGUAGE, OUTPUT_CHARSET environment variables.  */
+  /* Save LC_ALL, LANGUAGE environment variables.  */
 
   tmp = getenv ("LC_ALL");
   old_LC_ALL = (tmp != NULL ? xstrdup (tmp) : NULL);
@@ -1669,12 +1654,8 @@ get_title ()
   tmp = getenv ("LANGUAGE");
   old_LANGUAGE = (tmp != NULL ? xstrdup (tmp) : NULL);
 
-  tmp = getenv ("OUTPUT_CHARSET");
-  old_OUTPUT_CHARSET = (tmp != NULL ? xstrdup (tmp) : NULL);
-
   xsetenv ("LC_ALL", locale, 1);
   unsetenv ("LANGUAGE");
-  xsetenv ("OUTPUT_CHARSET", encoding, 1);
 
   if (setlocale (LC_ALL, "") == NULL)
     /* Nonexistent locale.  Use the English title.  */
@@ -1689,13 +1670,15 @@ get_title ()
       result = gettext (msgid);
       if (result != msgid && strcmp (result, msgid) != 0)
         /* Use the English and the foreign title.  */
-        result = xasprintf ("%s\n%s", english, result);
+        result = xasprintf ("%s\n%s", english,
+                            xstr_iconv (result, locale_charset (),
+                                        output_charset));
       else
         /* No translation found.  Use the English title.  */
         result = english;
     }
 
-  /* Restore LC_ALL, LANGUAGE, OUTPUT_CHARSET environment variables.  */
+  /* Restore LC_ALL, LANGUAGE environment variables.  */
 
   if (old_LC_ALL != NULL)
     xsetenv ("LC_ALL", old_LC_ALL, 1), free (old_LC_ALL);
@@ -1707,11 +1690,6 @@ get_title ()
   else
     unsetenv ("LANGUAGE");
 
-  if (old_OUTPUT_CHARSET != NULL)
-    xsetenv ("OUTPUT_CHARSET", old_OUTPUT_CHARSET, 1), free (old_OUTPUT_CHARSET);
-  else
-    unsetenv ("OUTPUT_CHARSET");
-
   setlocale (LC_ALL, "");
 
   return result;
@@ -1789,10 +1767,56 @@ subst_string_list (string_list_ty *slp,
 static msgdomain_list_ty *
 fill_header (msgdomain_list_ty *mdlp)
 {
+  /* Determine the desired encoding to the PO file.
+     If the POT file contains charset=UTF-8, it means that the POT file
+     contains non-ASCII characters, and we keep the UTF-8 encoding.
+     Otherwise, when the POT file is plain ASCII, we use the locale's
+     encoding.  */
+  bool was_utf8;
+  size_t k, j;
+
+  was_utf8 = false;
+  for (k = 0; k < mdlp->nitems; k++)
+    {
+      message_list_ty *mlp = mdlp->item[k]->messages;
+
+      if (mlp->nitems > 0)
+        {
+          message_ty *header_mp = NULL;
+
+          /* Search the header entry.  */
+          for (j = 0; j < mlp->nitems; j++)
+            if (is_header (mlp->item[j]) && !mlp->item[j]->obsolete)
+              {
+                header_mp = mlp->item[j];
+                break;
+              }
+
+          if (header_mp != NULL)
+            {
+              const char *header = header_mp->msgstr;
+              const char *old_field = get_field (header, "Content-Type");
+
+              if (old_field != NULL)
+                {
+                  const char *charsetstr = c_strstr (old_field, "charset=");
+                  if (charsetstr != NULL)
+                    {
+                      charsetstr += strlen ("charset=");
+                      if (c_strcasecmp (charsetstr, "UTF-8") == 0)
+                        was_utf8 = true;
+                    }
+                }
+            }
+        }
+    }
+
+  output_charset = (was_utf8 ? "UTF-8" : canonical_locale_charset ());
+
   /* Cache the strings filled in, for use when there are multiple domains
      and a header entry for each domain.  */
   const char *field_value[NFIELDS];
-  size_t k, j, i;
+  size_t i;
 
   for (i = 0; i < NFIELDS; i++)
     field_value[i] = NULL;
@@ -1875,6 +1899,8 @@ fill_header (msgdomain_list_ty *mdlp)
   return mdlp;
 }
 
+/* ------------------------------------------------------------------------- */
+
 
 /* Update the msgstr plural entries according to the nplurals count.  */
 static msgdomain_list_ty *
