@@ -20,7 +20,6 @@
 #endif
 #include <alloca.h>
 
-#include <getopt.h>
 #include <limits.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -34,6 +33,7 @@
 #include <textstyle.h>
 
 #include <error.h>
+#include "options.h"
 #include "noreturn.h"
 #include "closeout.h"
 #include "dir-list.h"
@@ -120,45 +120,6 @@ static bool update_mode = false;
 static const char *version_control_string;
 static const char *backup_suffix_string;
 
-/* Long options.  */
-static const struct option long_options[] =
-{
-  { "add-location", optional_argument, NULL, 'n' },
-  { "backup", required_argument, NULL, CHAR_MAX + 1 },
-  { "color", optional_argument, NULL, CHAR_MAX + 9 },
-  { "compendium", required_argument, NULL, 'C' },
-  { "directory", required_argument, NULL, 'D' },
-  { "escape", no_argument, NULL, 'E' },
-  { "for-msgfmt", no_argument, NULL, CHAR_MAX + 12 },
-  { "force-po", no_argument, &force_po, 1 },
-  { "help", no_argument, NULL, 'h' },
-  { "indent", no_argument, NULL, 'i' },
-  { "lang", required_argument, NULL, CHAR_MAX + 8 },
-  { "multi-domain", no_argument, NULL, 'm' },
-  { "no-escape", no_argument, NULL, 'e' },
-  { "no-fuzzy-matching", no_argument, NULL, 'N' },
-  { "no-location", no_argument, NULL, CHAR_MAX + 11 },
-  { "no-wrap", no_argument, NULL, CHAR_MAX + 4 },
-  { "output-file", required_argument, NULL, 'o' },
-  { "previous", no_argument, NULL, CHAR_MAX + 7 },
-  { "properties-input", no_argument, NULL, 'P' },
-  { "properties-output", no_argument, NULL, 'p' },
-  { "quiet", no_argument, NULL, 'q' },
-  { "sort-by-file", no_argument, NULL, 'F' },
-  { "sort-output", no_argument, NULL, 's' },
-  { "silent", no_argument, NULL, 'q' },
-  { "strict", no_argument, NULL, CHAR_MAX + 2 },
-  { "stringtable-input", no_argument, NULL, CHAR_MAX + 5 },
-  { "stringtable-output", no_argument, NULL, CHAR_MAX + 6 },
-  { "style", required_argument, NULL, CHAR_MAX + 10 },
-  { "suffix", required_argument, NULL, CHAR_MAX + 3 },
-  { "update", no_argument, NULL, 'U' },
-  { "verbose", no_argument, NULL, 'v' },
-  { "version", no_argument, NULL, 'V' },
-  { "width", required_argument, NULL, 'w' },
-  { NULL, 0, NULL, 0 }
-};
-
 
 struct statistics
 {
@@ -181,7 +142,6 @@ static msgdomain_list_ty *merge (const char *fn1, const char *fn2,
 int
 main (int argc, char **argv)
 {
-  int opt;
   bool do_help;
   bool do_version;
   char *output_file;
@@ -218,12 +178,52 @@ main (int argc, char **argv)
   output_file = NULL;
   color = NULL;
 
-  while ((opt = getopt_long (argc, argv, "C:D:eEFhimn:No:pPqsUvVw:",
-                             long_options, NULL))
-         != EOF)
+  /* Parse command line options.  */
+  BEGIN_ALLOW_OMITTING_FIELD_INITIALIZERS
+  static const struct program_option options[] =
+  {
+    { "add-location",       CHAR_MAX + 'n', optional_argument },
+    { NULL,                 'n',            no_argument       },
+    { "backup",             CHAR_MAX + 1,   required_argument },
+    { "color",              CHAR_MAX + 9,   optional_argument },
+    { "compendium",         'C',            required_argument },
+    { "directory",          'D',            required_argument },
+    { "escape",             'E',            no_argument       },
+    { "for-msgfmt",         CHAR_MAX + 12,  no_argument       },
+    { "force-po",           0,              no_argument,      &force_po, 1 },
+    { "help",               'h',            no_argument       },
+    { "indent",             'i',            no_argument       },
+    { "lang",               CHAR_MAX + 8,   required_argument },
+    { "multi-domain",       'm',            no_argument       },
+    { "no-escape",          'e',            no_argument       },
+    { "no-fuzzy-matching",  'N',            no_argument       },
+    { "no-location",        CHAR_MAX + 11,  no_argument       },
+    { "no-wrap",            CHAR_MAX + 4,   no_argument       },
+    { "output-file",        'o',            required_argument },
+    { "previous",           CHAR_MAX + 7,   no_argument       },
+    { "properties-input",   'P',            no_argument       },
+    { "properties-output",  'p',            no_argument       },
+    { "quiet",              'q',            no_argument       },
+    { "sort-by-file",       'F',            no_argument       },
+    { "sort-output",        's',            no_argument       },
+    { "silent",             'q',            no_argument       },
+    { "strict",             CHAR_MAX + 2,   no_argument       },
+    { "stringtable-input",  CHAR_MAX + 5,   no_argument       },
+    { "stringtable-output", CHAR_MAX + 6,   no_argument       },
+    { "style",              CHAR_MAX + 10,  required_argument },
+    { "suffix",             CHAR_MAX + 3,   required_argument },
+    { "update",             'U',            no_argument       },
+    { "verbose",            'v',            no_argument       },
+    { "version",            'V',            no_argument       },
+    { "width",              'w',            required_argument },
+  };
+  END_ALLOW_OMITTING_FIELD_INITIALIZERS
+  start_options (argc, argv, options, MOVE_OPTIONS_FIRST, 0);
+  int opt;
+  while ((opt = get_next_option ()) != -1)
     switch (opt)
       {
-      case '\0':                /* Long option.  */
+      case '\0':                /* Long option with key == 0.  */
         break;
 
       case 'C':
@@ -258,7 +258,8 @@ main (int argc, char **argv)
         multi_domain_mode = true;
         break;
 
-      case 'n':
+      case 'n':            /* -n */
+      case CHAR_MAX + 'n': /* --add-location[={full|yes|file|never|no}] */
         if (handle_filepos_comment_option (optarg))
           usage (EXIT_FAILURE);
         break;
@@ -1409,12 +1410,6 @@ message_merge (message_ty *def, message_ty *ref, bool force_fuzzy,
 
   result->do_wrap = ref->do_wrap;
 
-  {
-    size_t i;
-    for (i = 0; i < NSYNTAXCHECKS; i++)
-      result->do_syntax_check[i] = ref->do_syntax_check[i];
-  }
-
   /* Insert previous msgid, commented out with "#|".
      Do so only when --previous is specified, for backward compatibility.
      Since the "previous msgid" represents the original msgid that led to
