@@ -1,5 +1,5 @@
 /* xgettext C/C++/ObjectiveC backend.
-   Copyright (C) 1995-2024 Free Software Foundation, Inc.
+   Copyright (C) 1995-2025 Free Software Foundation, Inc.
 
    This file was written by Peter Miller <millerp@canb.auug.org.au>
 
@@ -30,6 +30,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define SB_NO_APPENDF
 #include <error.h>
 #include "attribute.h"
 #include "message.h"
@@ -1345,8 +1346,8 @@ phase5_get (token_ty *tp)
               case '"':
                 {
                   string_desc_t contents = sb_contents (&buffer);
-                  const char *buf = string_desc_data (contents);
-                  size_t buflen = string_desc_length (contents);
+                  const char *buf = sd_data (contents);
+                  size_t buflen = sd_length (contents);
 
                   /* Recognize C11 / C++11 string literals.
                      See (for C) ISO 9899:2011 section 6.4.5
@@ -1463,14 +1464,14 @@ phase5_get (token_ty *tp)
                                 break;
 
                               /* Update the state.  */
-                              string_desc_t contents = sb_contents (&buffer);
-                              const char *buf = string_desc_data (contents);
-                              size_t buflen = string_desc_length (contents);
-                              if (c == (state < buflen ? buf[state] : '"'))
+                              string_desc_t raw_contents = sb_contents (&buffer);
+                              const char *raw_buf = sd_data (raw_contents);
+                              size_t raw_buflen = sd_length (raw_contents);
+                              if (c == (state < raw_buflen ? raw_buf[state] : '"'))
                                 {
-                                  if (state < buflen)
+                                  if (state < raw_buflen)
                                     state++;
-                                  else /* state == buflen && c == '"' */
+                                  else /* state == raw_buflen && c == '"' */
                                     {
                                       /* Finished parsing the string.  */
                                       sb_free (&buffer);
@@ -1484,10 +1485,10 @@ phase5_get (token_ty *tp)
                                 {
                                   int i;
 
-                                  /* None of the bytes buf[0]...buf[state-1]
+                                  /* None of the bytes raw_buf[0]...raw_buf[state-1]
                                      can be ')'.  */
                                   for (i = 0; i < state; i++)
-                                    mixed_string_buffer_append_char (&msb, buf[i]);
+                                    mixed_string_buffer_append_char (&msb, raw_buf[i]);
 
                                   /* But c may be ')'.  */
                                   if (c == ')')
@@ -1650,11 +1651,10 @@ phase5_get (token_ty *tp)
                        can be part of a number token.  It's called a "digit
                        separator".  See ISO C 23 § 6.4.4.1 and § 6.4.4.2.  */
                     string_desc_t contents = sb_contents (&buffer);
-                    if (string_desc_length (contents) > 0)
+                    if (sd_length (contents) > 0)
                       {
                         char prev =
-                          string_desc_char_at (contents,
-                                               string_desc_length (contents) - 1);
+                          sd_char_at (contents, sd_length (contents) - 1);
                         if ((prev >= '0' && prev <= '9')
                             || (prev >= 'A' && prev <= 'F')
                             || (prev >= 'a' && prev <= 'f'))
