@@ -1,6 +1,5 @@
 /* xgettext C/C++/ObjectiveC backend.
-   Copyright (C) 2001-2003, 2006, 2009, 2015-2016 Free Software Foundation,
-   Inc.
+   Copyright (C) 2001-2025 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -14,13 +13,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 
 #include <stdio.h>
 
 #include "message.h"
-#include "xgettext.h"
+#include "xg-arglist-context.h"
 
 
 #ifdef __cplusplus
@@ -42,22 +41,18 @@ extern "C" {
   { "m",      "ObjectiveC" },                                           \
 
 #define SCANNERS_C \
-  { "C",                extract_c,                                      \
+  { "C",                extract_c, NULL,                                \
                         &flag_table_c,                                  \
-                        &formatstring_c, NULL,                          \
-                        &literalstring_c },                             \
-  { "C++",              extract_cxx,                                    \
+                        &formatstring_c, NULL },                        \
+  { "C++",              extract_cxx, NULL,                              \
                         &flag_table_c,                                  \
-                        &formatstring_c, NULL,                          \
-                        &literalstring_c },                             \
-  { "ObjectiveC",       extract_objc,                                   \
+                        &formatstring_c, &formatstring_cplusplus_brace }, \
+  { "ObjectiveC",       extract_objc, NULL,                             \
                         &flag_table_objc,                               \
-                        &formatstring_c, &formatstring_objc,            \
-                        &literalstring_c },                             \
-  { "GCC-source",       extract_c,                                      \
+                        &formatstring_c, &formatstring_objc },          \
+  { "GCC-source",       extract_c, NULL,                                \
                         &flag_table_gcc_internal,                       \
-                        &formatstring_gcc_internal, &formatstring_gfc_internal, \
-                        &literalstring_c },                             \
+                        &formatstring_gcc_internal, NULL },             \
 
 /* Scan a C file and add its translatable strings to mdlp.  */
 extern void extract_c (FILE *fp, const char *real_filename,
@@ -93,9 +88,6 @@ extern void init_flag_table_gcc_internal (void);
 extern void init_flag_table_kde (void);
 
 
-extern struct literalstring_parser literalstring_c;
-
-
 #ifdef __cplusplus
 }
 #endif
