@@ -1,5 +1,5 @@
 /* xgettext Vala backend.
-   Copyright (C) 2013-2014, 2018-2020 Free Software Foundation, Inc.
+   Copyright (C) 2013-2025 Free Software Foundation, Inc.
 
    This file was written by Daiki Ueno <ueno@gnu.org>, 2013.
 
@@ -30,6 +30,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define SB_NO_APPENDF
+#include <error.h>
 #include "attribute.h"
 #include "message.h"
 #include "rc-str-list.h"
@@ -41,10 +43,10 @@
 #include "xg-arglist-callshape.h"
 #include "xg-arglist-parser.h"
 #include "xg-message.h"
-#include "error.h"
-#include "error-progname.h"
+#include "if-error.h"
 #include "xalloc.h"
 #include "xvasprintf.h"
+#include "string-buffer.h"
 #include "mem-hash-map.h"
 #include "po-charset.h"
 #include "gettext.h"
@@ -54,8 +56,14 @@
 #define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
 
 /* The Vala syntax is defined in the Vala Reference Manual
-   https://www.vala-project.org/doc/vala/.
-   See also vala/valascanner.vala.  */
+   https://gnome.pages.gitlab.gnome.org/vala/manual/index.html.
+   See also vala/valascanner.vala.
+
+   It supports string formatting through functions and methods, namely
+   through the string.printf and string.vprintf methods:
+   <https://valadoc.org/glib-2.0/string.printf.html>
+   <https://valadoc.org/glib-2.0/string.vprintf.html>
+ */
 
 /* ====================== Keyword set customization.  ====================== */
 
@@ -128,24 +136,70 @@ init_keywords ()
 void
 init_flag_table_vala ()
 {
-  xgettext_record_flag ("dgettext:2:pass-c-format");
-  xgettext_record_flag ("dcgettext:2:pass-c-format");
-  xgettext_record_flag ("ngettext:1:pass-c-format");
-  xgettext_record_flag ("ngettext:2:pass-c-format");
-  xgettext_record_flag ("dngettext:2:pass-c-format");
-  xgettext_record_flag ("dngettext:3:pass-c-format");
-  xgettext_record_flag ("dpgettext:2:pass-c-format");
-  xgettext_record_flag ("dpgettext2:3:pass-c-format");
-  xgettext_record_flag ("_:1:pass-c-format");
-  xgettext_record_flag ("Q_:1:pass-c-format");
-  xgettext_record_flag ("N_:1:pass-c-format");
-  xgettext_record_flag ("NC_:2:pass-c-format");
-
   /* Vala leaves string formatting to Glib functions and thus the
      format string is exactly same as C.  See also
      vapi/glib-2.0.vapi.  */
-  xgettext_record_flag ("printf:1:c-format");
-  xgettext_record_flag ("vprintf:1:c-format");
+
+  xgettext_record_flag ("dgettext:2:pass-c-format!Vala");
+  xgettext_record_flag ("dcgettext:2:pass-c-format!Vala");
+  xgettext_record_flag ("ngettext:1:pass-c-format!Vala");
+  xgettext_record_flag ("ngettext:2:pass-c-format!Vala");
+  xgettext_record_flag ("dngettext:2:pass-c-format!Vala");
+  xgettext_record_flag ("dngettext:3:pass-c-format!Vala");
+  xgettext_record_flag ("dpgettext:2:pass-c-format!Vala");
+  xgettext_record_flag ("dpgettext2:3:pass-c-format!Vala");
+  xgettext_record_flag ("_:1:pass-c-format!Vala");
+  xgettext_record_flag ("Q_:1:pass-c-format!Vala");
+  xgettext_record_flag ("N_:1:pass-c-format!Vala");
+  xgettext_record_flag ("NC_:2:pass-c-format!Vala");
+
+  /* In Vala, vprintf does not exist as a function, only as a method.
+     https://valadoc.org/?q=printf provides this list:
+
+     Method                                             Arguments
+
+     Posix.FILE.printf (posix)                          (string format, ...)
+     Posix.printf (posix)                               (string format, ...)
+     Alsa.Output.printf (alsa)                          (string format, ...)
+     Dazzle.Path.printf (libdazzle-1.0)                 ()
+     Dazzle.ShortcutChordTable.printf (libdazzle-1.0)   ()
+     string.printf (glib-2.0)                           (...)
+     GLib.FileStream.printf (glib-2.0)                  (string format, ...)
+     GLib.StringBuilder.printf (glib-2.0)               (string format, ...)
+     GLib.Variant.Variant.printf (glib-2.0)             (string format, ...)
+     FastCGI.FileStream.printf (fcgi)                   (string format, ...)
+     FastCGI.Stream.printf (fcgi)                       (string format, ...)
+     Purple.Stringref.printf (purple)                   (string format)
+     Gsl.Stream.printf (gsl)                            (string label, string file, int line, string reason)
+     Gsf.Output.printf (libgsf-1)                       (string format, ...)
+     GLib.OutputStream.printf (gio-2.0)                 (out size_t bytes_written, Cancellable? cancellable, string format, ...)
+     TokyoCabinet.XString.printf (tokyocabinet)         (string format, ...)
+     ZLib.GZFileStream.printf (zlib)                    (string format, ...)
+
+     Therefore, whenever the first argument is a string, it may be a format
+     string or a plain string.  We don't know.  Therefore we cannot enable
+     this flag.  Recognition of format strings that occur as a first argument
+     therefore relies on the heuristics.  */
+  /* Override the effect of
+       xgettext_record_flag ("printf:1:c-format");
+     in x-c.c.  */
+  xgettext_record_flag ("printf:1:undecided-c-format!Vala");
+
+  /* In Vala, vprintf does not exist as a function, only as a method.
+     https://valadoc.org/?q=vprintf provides this list:
+
+     Method                                             Arguments
+     string.vprintf (glib-2.0)                          (va_list args)
+     GLib.FileStream.vprintf (glib-2.0)                 (string format, va_list args)
+     GLib.StringBuilder.vprintf (glib-2.0)              (string format, va_list args)
+     FastCGI.FileStream.vprintf (fcgi)                  (string format, va_list args)
+     FastCGI.Stream.vprintf (fcgi)                      (string format, va_list args)
+     Gsf.Output.vprintf (libgsf-1)                      (string format, va_list args)
+     GLib.OutputStream.vprintf (gio-2.0)                (out size_t bytes_written, Cancellable? cancellable, string format, va_list args)
+
+     Therefore, whenever the first argument is a string, it must be a format
+     string.  */
+  xgettext_record_flag ("vprintf:1:c-format!Vala");
 }
 
 
@@ -344,10 +398,11 @@ enum token_type_ty
   token_type_rparen,                    /* ) */
   token_type_lbrace,                    /* { */
   token_type_rbrace,                    /* } */
-  token_type_assign,                    /* = += -= *= /= %= <<= >>= &= |= ^= */
+  token_type_assign,                    /* = */
+  token_type_compound_assign,           /* += -= *= /= %= <<= >>= &= |= ^= */
   token_type_return,                    /* return */
   token_type_plus,                      /* + */
-  token_type_arithmetic_operator,       /* - * / % << >> & | ^ */
+  token_type_arithmetic_operator,       /* - * / % << >> & | ^ ~ */
   token_type_equality_test_operator,    /* == < > >= <= != */
   token_type_logic_operator,            /* ! && || */
   token_type_comma,                     /* , */
@@ -357,6 +412,7 @@ enum token_type_ty
   token_type_string_literal,            /* "abc" */
   token_type_string_template,           /* @"abc" */
   token_type_regex_literal,             /* /.../ */
+  token_type_semicolon,                 /* ; */
   token_type_symbol,                    /* if else etc. */
   token_type_other
 };
@@ -386,15 +442,14 @@ free_token (token_ty *tp)
 }
 
 
-/* Return value of phase7_getc when EOF is reached.  */
-#define P7_EOF (-1)
-#define P7_STRING_END (-2)
+/* Return value of get_string_element when EOF is reached.  */
+#define SE_EOF (-1)
 
 /* Replace escape sequences within character strings with their single
    character equivalents.  */
-#define P7_QUOTES (-3)
-#define P7_QUOTE (-4)
-#define P7_NEWLINE (-5)
+#define SE_QUOTES (-3)
+#define SE_QUOTE (-4)
+#define SE_NEWLINE (-5)
 
 /* Convert an UTF-16 or UTF-32 code point to a return value that can be
    distinguished from a single-byte return value.  */
@@ -410,13 +465,16 @@ free_token (token_ty *tp)
 
 
 static int
-phase7_getc ()
+get_string_element ()
 {
-  int c, n, j;
+  int c, j;
 
   /* Use phase 1, because phase 2 elides comments.  */
   c = phase1_getc ();
 
+  if (c == EOF)
+    return SE_EOF;
+
   /* Return a magic newline indicator, so that we can distinguish
      between the user requesting a newline in the string (e.g. using
      "\n" or "\012") from the user failing to terminate the string or
@@ -435,12 +493,12 @@ phase7_getc ()
      you may not embed newlines in character constants; try it, you get
      a useful diagnostic.  --PMiller  */
   if (c == '\n')
-    return P7_NEWLINE;
+    return SE_NEWLINE;
 
   if (c == '"')
-    return P7_QUOTES;
+    return SE_QUOTES;
   if (c == '\'')
-    return P7_QUOTE;
+    return SE_QUOTE;
   if (c != '\\')
     return c;
   c = phase1_getc ();
@@ -487,55 +545,79 @@ phase7_getc ()
         case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
           break;
         }
-      n = 0;
-      for (;;)
-        {
-          switch (c)
-            {
-            default:
-              phase1_ungetc (c);
-              return n;
+      {
+        int n;
+        bool overflow;
 
-            case '0': case '1': case '2': case '3': case '4':
-            case '5': case '6': case '7': case '8': case '9':
-              n = n * 16 + c - '0';
+        n = 0;
+        overflow = false;
+
+        for (;;)
+          {
+            switch (c)
+              {
+              default:
+                phase1_ungetc (c);
+                if (overflow)
+                  if_error (IF_SEVERITY_WARNING,
+                            logical_file_name, line_number, (size_t)(-1), false,
+                            _("hexadecimal escape sequence out of range"));
+                return n;
+
+              case '0': case '1': case '2': case '3': case '4':
+              case '5': case '6': case '7': case '8': case '9':
+                if (n < 0x100 / 16)
+                  n = n * 16 + c - '0';
+                else
+                  overflow = true;
               break;
 
-            case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
-              n = n * 16 + 10 + c - 'A';
-              break;
+              case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
+                if (n < 0x100 / 16)
+                  n = n * 16 + 10 + c - 'A';
+                else
+                  overflow = true;
+                break;
 
-            case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
-              n = n * 16 + 10 + c - 'a';
-              break;
-            }
-          c = phase1_getc ();
-        }
-      return n;
+              case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
+                if (n < 0x100 / 16)
+                  n = n * 16 + 10 + c - 'a';
+                else
+                  overflow = true;
+                break;
+              }
+            c = phase1_getc ();
+          }
+      }
 
     case '0':
-      n = 0;
-      for (j = 0; j < 3; ++j)
-        {
-          n = n * 8 + c - '0';
-          c = phase1_getc ();
-          switch (c)
-            {
-            default:
-              break;
+      {
+        int n;
 
-            case '0': case '1': case '2': case '3':
-            case '4': case '5': case '6': case '7':
-              continue;
-            }
-          break;
-        }
-      phase1_ungetc (c);
-      return n;
+        n = 0;
+        for (j = 0; j < 3; ++j)
+          {
+            n = n * 8 + c - '0';
+            c = phase1_getc ();
+            switch (c)
+              {
+              default:
+                break;
+
+              case '0': case '1': case '2': case '3':
+              case '4': case '5': case '6': case '7':
+                continue;
+              }
+            break;
+          }
+        phase1_ungetc (c);
+        return n;
+      }
 
     case 'u':
       {
         unsigned char buf[8];
+        int n;
 
         n = 0;
         for (j = 0; j < 4; j++)
@@ -563,10 +645,9 @@ phase7_getc ()
         if (n < 0x110000)
           return UNICODE (n);
 
-        error_with_progname = false;
-        error (0, 0, _("%s:%d: warning: invalid Unicode character"),
-               logical_file_name, line_number);
-        error_with_progname = true;
+        if_error (IF_SEVERITY_WARNING,
+                  logical_file_name, line_number, (size_t)(-1), false,
+                  _("invalid Unicode character"));
 
         while (--j >= 0)
           phase1_ungetc (buf[j]);
@@ -578,7 +659,7 @@ phase7_getc ()
 
 
 static void
-phase7_ungetc (int c)
+unget_string_element (int c)
 {
   phase1_ungetc (c);
 }
@@ -612,11 +693,9 @@ phase3_scan_regex ()
           }
         if (c == EOF)
           {
-            error_with_progname = false;
-            error (0, 0,
-                   _("%s:%d: warning: regular expression literal terminated too early"),
-                   logical_file_name, line_number);
-            error_with_progname = true;
+            if_error (IF_SEVERITY_WARNING,
+                      logical_file_name, line_number, (size_t)(-1), false,
+                      _("regular expression literal terminated too early"));
             return;
           }
       }
@@ -629,22 +708,8 @@ phase3_scan_regex ()
 static void
 phase3_get (token_ty *tp)
 {
-  static char *buffer;
-  static int bufmax;
-  int bufpos;
-
 #undef APPEND
-#define APPEND(c)                               \
-  do                                            \
-    {                                           \
-      if (bufpos >= bufmax)                     \
-        {                                       \
-          bufmax = 2 * bufmax + 10;             \
-          buffer = xrealloc (buffer, bufmax);   \
-        }                                       \
-      buffer[bufpos++] = c;                     \
-    }                                           \
-  while (0)
+#define APPEND(c) sb_xappend1 (&buffer, (c))
 
   if (phase3_pushback_length)
     {
@@ -696,42 +761,48 @@ phase3_get (token_ty *tp)
         case 'h': case 'i': case 'j': case 'k': case 'l': case 'm': case 'n':
         case 'o': case 'p': case 'q': case 'r': case 's': case 't': case 'u':
         case 'v': case 'w': case 'x': case 'y': case 'z':
-          bufpos = 0;
-          for (;;)
-            {
-              APPEND (c);
-              c = phase2_getc ();
-              switch (c)
-                {
-                case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
-                case 'G': case 'H': case 'I': case 'J': case 'K': case 'L':
-                case 'M': case 'N': case 'O': case 'P': case 'Q': case 'R':
-                case 'S': case 'T': case 'U': case 'V': case 'W': case 'X':
-                case 'Y': case 'Z':
-                case '_':
-                case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
-                case 'g': case 'h': case 'i': case 'j': case 'k': case 'l':
-                case 'm': case 'n': case 'o': case 'p': case 'q': case 'r':
-                case 's': case 't': case 'u': case 'v': case 'w': case 'x':
-                case 'y': case 'z':
-                case '0': case '1': case '2': case '3': case '4':
-                case '5': case '6': case '7': case '8': case '9':
-                  continue;
+          {
+            struct string_buffer buffer;
+            sb_init (&buffer);
+            for (;;)
+              {
+                APPEND (c);
+                c = phase2_getc ();
+                switch (c)
+                  {
+                  case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
+                  case 'G': case 'H': case 'I': case 'J': case 'K': case 'L':
+                  case 'M': case 'N': case 'O': case 'P': case 'Q': case 'R':
+                  case 'S': case 'T': case 'U': case 'V': case 'W': case 'X':
+                  case 'Y': case 'Z':
+                  case '_':
+                  case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
+                  case 'g': case 'h': case 'i': case 'j': case 'k': case 'l':
+                  case 'm': case 'n': case 'o': case 'p': case 'q': case 'r':
+                  case 's': case 't': case 'u': case 'v': case 'w': case 'x':
+                  case 'y': case 'z':
+                  case '0': case '1': case '2': case '3': case '4':
+                  case '5': case '6': case '7': case '8': case '9':
+                    continue;
 
-                default:
-                  phase2_ungetc (c);
-                  break;
-                }
-              break;
-            }
-          APPEND (0);
-          if (strcmp (buffer, "return") == 0)
-            tp->type = last_token_type = token_type_return;
-          else
-            {
-              tp->string = xstrdup (buffer);
-              tp->type = last_token_type = token_type_symbol;
+                  default:
+                    phase2_ungetc (c);
+                    break;
+                  }
+                break;
             }
+            const char *contents = sb_xcontents_c (&buffer);
+            if (strcmp (contents, "return") == 0)
+              {
+                sb_free (&buffer);
+                tp->type = last_token_type = token_type_return;
+              }
+            else
+              {
+                tp->string = sb_xdupfree_c (&buffer);
+                tp->type = last_token_type = token_type_symbol;
+              }
+          }
           return;
 
         case '.':
@@ -756,63 +827,65 @@ phase3_get (token_ty *tp)
           /* The preprocessing number token is more "generous" than the C
              number tokens.  This is mostly due to token pasting (another
              thing we can ignore here).  */
-          bufpos = 0;
-          for (;;)
-            {
-              APPEND (c);
-              c = phase2_getc ();
-              switch (c)
-                {
-                case 'e':
-                case 'E':
-                  APPEND (c);
-                  c = phase2_getc ();
-                  if (c != '+' && c != '-')
-                    {
-                      phase2_ungetc (c);
-                      break;
-                    }
-                  continue;
+          {
+            struct string_buffer buffer;
+            sb_init (&buffer);
+            for (;;)
+              {
+                APPEND (c);
+                c = phase2_getc ();
+                switch (c)
+                  {
+                  case 'e':
+                  case 'E':
+                    APPEND (c);
+                    c = phase2_getc ();
+                    if (c != '+' && c != '-')
+                      {
+                        phase2_ungetc (c);
+                        break;
+                      }
+                    continue;
 
-                case 'A': case 'B': case 'C': case 'D':           case 'F':
-                case 'G': case 'H': case 'I': case 'J': case 'K': case 'L':
-                case 'M': case 'N': case 'O': case 'P': case 'Q': case 'R':
-                case 'S': case 'T': case 'U': case 'V': case 'W': case 'X':
-                case 'Y': case 'Z':
-                case 'a': case 'b': case 'c': case 'd':           case 'f':
-                case 'g': case 'h': case 'i': case 'j': case 'k': case 'l':
-                case 'm': case 'n': case 'o': case 'p': case 'q': case 'r':
-                case 's': case 't': case 'u': case 'v': case 'w': case 'x':
-                case 'y': case 'z':
-                case '0': case '1': case '2': case '3': case '4':
-                case '5': case '6': case '7': case '8': case '9':
-                case '.':
-                  continue;
+                  case 'A': case 'B': case 'C': case 'D':           case 'F':
+                  case 'G': case 'H': case 'I': case 'J': case 'K': case 'L':
+                  case 'M': case 'N': case 'O': case 'P': case 'Q': case 'R':
+                  case 'S': case 'T': case 'U': case 'V': case 'W': case 'X':
+                  case 'Y': case 'Z':
+                  case 'a': case 'b': case 'c': case 'd':           case 'f':
+                  case 'g': case 'h': case 'i': case 'j': case 'k': case 'l':
+                  case 'm': case 'n': case 'o': case 'p': case 'q': case 'r':
+                  case 's': case 't': case 'u': case 'v': case 'w': case 'x':
+                  case 'y': case 'z':
+                  case '0': case '1': case '2': case '3': case '4':
+                  case '5': case '6': case '7': case '8': case '9':
+                  case '.':
+                    continue;
 
-                default:
-                  phase2_ungetc (c);
-                  break;
-                }
-              break;
-            }
-          APPEND (0);
-          tp->type = last_token_type = token_type_number;
+                  default:
+                    phase2_ungetc (c);
+                    break;
+                  }
+                break;
+              }
+            sb_free (&buffer);
+            tp->type = last_token_type = token_type_number;
+          }
           return;
 
         case '\'':
           for (;;)
             {
-              c = phase7_getc ();
-              if (c == P7_NEWLINE)
+              c = get_string_element ();
+              if (c == SE_NEWLINE)
                 {
-                  error_with_progname = false;
-                  error (0, 0, _("%s:%d: warning: unterminated character constant"),
-                         logical_file_name, line_number - 1);
-                  error_with_progname = true;
-                  phase7_ungetc ('\n');
+                  if_error (IF_SEVERITY_WARNING,
+                            logical_file_name, line_number - 1, (size_t)(-1), false,
+                            _("unterminated character constant"));
+                  unget_string_element ('\n');
                   break;
                 }
-              if (c == EOF || c == P7_QUOTE)
+              if (c == SE_EOF || c == SE_QUOTE)
                 break;
             }
           tp->type = last_token_type = token_type_character_constant;
@@ -850,21 +923,23 @@ phase3_get (token_ty *tp)
         case '"':
           {
             struct mixed_string_buffer msb;
-            int c2 = phase1_getc ();
+            {
+              int c2 = phase1_getc ();
 
-            if (c2 == '"')
-              {
-                int c3 = phase1_getc ();
-                if (c3 == '"')
-                  verbatim = true;
-                else
-                  {
-                    phase1_ungetc (c3);
-                    phase1_ungetc (c2);
-                  }
-              }
-            else
-              phase2_ungetc (c2);
+              if (c2 == '"')
+                {
+                  int c3 = phase1_getc ();
+                  if (c3 == '"')
+                    verbatim = true;
+                  else
+                    {
+                      phase1_ungetc (c3);
+                      phase1_ungetc (c2);
+                    }
+                }
+              else
+                phase2_ungetc (c2);
+            }
 
             /* Start accumulating the string.  */
             mixed_string_buffer_init (&msb, lc_string,
@@ -896,26 +971,24 @@ phase3_get (token_ty *tp)
             else
               for (;;)
                 {
-                  c = phase7_getc ();
+                  c = get_string_element ();
 
                   /* Keep line_number in sync.  */
                   msb.line_number = line_number;
 
-                  if (c == P7_NEWLINE)
+                  if (c == SE_NEWLINE)
                     {
-                      error_with_progname = false;
-                      error (0, 0,
-                             _("%s:%d: warning: unterminated string literal"),
-                             logical_file_name, line_number - 1);
-                      error_with_progname = true;
-                      phase7_ungetc ('\n');
+                      if_error (IF_SEVERITY_WARNING,
+                                logical_file_name, line_number - 1, (size_t)(-1), false,
+                                _("unterminated string literal"));
+                      unget_string_element ('\n');
                       break;
                     }
-                  if (c == P7_QUOTES)
+                  if (c == SE_QUOTES)
                     break;
-                  if (c == EOF)
+                  if (c == SE_EOF)
                     break;
-                  if (c == P7_QUOTE)
+                  if (c == SE_QUOTE)
                     c = '\'';
                   if (IS_UNICODE (c))
                     {
@@ -949,6 +1022,7 @@ phase3_get (token_ty *tp)
             case token_type_lparen:
             case token_type_lbrace:
             case token_type_assign:
+            case token_type_compound_assign:
             case token_type_return:
             case token_type_plus:
             case token_type_arithmetic_operator:
@@ -964,7 +1038,7 @@ phase3_get (token_ty *tp)
               {
                 int c2 = phase2_getc ();
                 if (c2 == '=')
-                  tp->type = last_token_type = token_type_assign;
+                  tp->type = last_token_type = token_type_compound_assign;
                 else
                   {
                     phase2_ungetc (c2);
@@ -1000,7 +1074,7 @@ phase3_get (token_ty *tp)
                 tp->type = last_token_type = token_type_other;
                 break;
               case '=':
-                tp->type = last_token_type = token_type_assign;
+                tp->type = last_token_type = token_type_compound_assign;
                 break;
               default:
                 phase2_ungetc (c2);
@@ -1019,7 +1093,7 @@ phase3_get (token_ty *tp)
                 tp->type = last_token_type = token_type_other;
                 break;
               case '=':
-                tp->type = last_token_type = token_type_assign;
+                tp->type = last_token_type = token_type_compound_assign;
                 break;
               default:
                 phase2_ungetc (c2);
@@ -1029,20 +1103,37 @@ phase3_get (token_ty *tp)
             return;
           }
 
+        case '*':
+          {
+            int c2 = phase2_getc ();
+            if (c2 == '=')
+              tp->type = last_token_type = token_type_compound_assign;
+            else
+              {
+                phase2_ungetc (c2);
+                tp->type = last_token_type = token_type_arithmetic_operator;
+              }
+            return;
+          }
+
         case '%':
         case '^':
           {
             int c2 = phase2_getc ();
             if (c2 == '=')
-	      tp->type = last_token_type = token_type_assign;
+	      tp->type = last_token_type = token_type_compound_assign;
             else
               {
                 phase2_ungetc (c2);
-                tp->type = last_token_type = token_type_logic_operator;
+                tp->type = last_token_type = token_type_arithmetic_operator;
               }
             return;
           }
 
+        case '~':
+          tp->type = last_token_type = token_type_arithmetic_operator;
+          return;
+
         case '=':
           {
             int c2 = phase2_getc ();
@@ -1085,7 +1176,7 @@ phase3_get (token_ty *tp)
               {
                 int c3 = phase2_getc ();
                 if (c3 == '=')
-                  tp->type = last_token_type = token_type_assign;
+                  tp->type = last_token_type = token_type_compound_assign;
                 else
                   {
                     phase2_ungetc (c2);
@@ -1116,7 +1207,7 @@ phase3_get (token_ty *tp)
             if (c2 == c)
 	      tp->type = last_token_type = token_type_logic_operator;
             else if (c2 == '=')
-	      tp->type = last_token_type = token_type_assign;
+	      tp->type = last_token_type = token_type_compound_assign;
             else
               {
                 phase2_ungetc (c2);
@@ -1138,6 +1229,10 @@ phase3_get (token_ty *tp)
             return;
           }
 
+        case ';':
+          tp->type = last_token_type = token_type_semicolon;
+          return;
+
         default:
           tp->type = last_token_type = token_type_other;
           return;
@@ -1158,11 +1253,20 @@ phase3_unget (token_ty *tp)
 }
 
 
-/* String concatenation with '+'.  */
+/* 4. String concatenation with '+'.  */
+
+static token_ty phase4_pushback[2];
+static int phase4_pushback_length;
 
 static void
-x_vala_lex (token_ty *tp)
+phase4_get (token_ty *tp)
 {
+  if (phase4_pushback_length)
+    {
+      *tp = phase4_pushback[--phase4_pushback_length];
+      return;
+    }
+
   phase3_get (tp);
   if (tp->type == token_type_string_literal)
     {
@@ -1195,6 +1299,31 @@ x_vala_lex (token_ty *tp)
     }
 }
 
+static void
+phase4_unget (token_ty *tp)
+{
+  if (tp->type != token_type_eof)
+    {
+      if (phase4_pushback_length == SIZEOF (phase4_pushback))
+        abort ();
+      phase4_pushback[phase4_pushback_length++] = *tp;
+    }
+}
+
+
+static void
+x_vala_lex (token_ty *tp)
+{
+  phase4_get (tp);
+}
+
+/* Supports 2 tokens of pushback.  */
+static void
+x_vala_unlex (token_ty *tp)
+{
+  phase4_unget (tp);
+}
+
 
 /* ========================= Extracting strings.  ========================== */
 
@@ -1203,6 +1332,13 @@ x_vala_lex (token_ty *tp)
 static flag_context_list_table_ty *flag_context_list_table;
 
 
+/* Maximum supported nesting depth.  */
+#define MAX_NESTING_DEPTH 1000
+
+/* Current nesting depth.  */
+static int nesting_depth;
+
+
 /* The file is broken into tokens.  Scan the token stream, looking for
    a keyword, followed by a left paren, followed by a string.  When we
    see this sequence, we have something to remember.  We assume we are
@@ -1218,14 +1354,15 @@ static flag_context_list_table_ty *flag_context_list_table;
    We use recursion because the arguments before msgid or between msgid
    and msgid_plural can contain subexpressions of the same form.  */
 
-/* Extract messages until the next balanced closing parenthesis or bracket.
+/* Extract messages until the next balanced closing parenthesis or bracket
+   or the next semicolon.
    Extracted messages are added to MLP.
    DELIM can be either token_type_rparen or token_type_rbracket, or
    token_type_eof to accept both.
    Return true upon eof, false upon closing parenthesis or bracket.  */
 static bool
 extract_balanced (message_list_ty *mlp, token_type_ty delim,
-                  flag_context_ty outer_context,
+                  flag_region_ty *outer_region,
                   flag_context_list_iterator_ty context_iter,
                   struct arglist_parser *argparser)
 {
@@ -1239,9 +1376,10 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
   flag_context_list_iterator_ty next_context_iter =
     passthrough_context_list_iterator;
   /* Current context.  */
-  flag_context_ty inner_context =
-    inherited_context (outer_context,
-                       flag_context_list_iterator_advance (&context_iter));
+  flag_context_ty curr_context =
+    flag_context_list_iterator_advance (&context_iter);
+  /* Current region.  */
+  flag_region_ty *inner_region = new_sub_region (outer_region, curr_context);
 
   /* Start state is 0.  */
   state = 0;
@@ -1277,14 +1415,42 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
           continue;
 
         case token_type_lparen:
+          if (++nesting_depth > MAX_NESTING_DEPTH)
+            if_error (IF_SEVERITY_FATAL_ERROR,
+                      logical_file_name, line_number, (size_t)(-1), false,
+                      _("too many open parentheses"));
           if (extract_balanced (mlp, token_type_rparen,
-                                inner_context, next_context_iter,
+                                inner_region, next_context_iter,
                                 arglist_parser_alloc (mlp,
                                                       state ? next_shapes : NULL)))
             {
               arglist_parser_done (argparser, arg);
+              unref_region (inner_region);
               return true;
             }
+          nesting_depth--;
+          /* Test whether the next tokens are '.' and 'printf' or 'vprintf'.  */
+          {
+            token_ty token2;
+            x_vala_lex (&token2);
+            if (token2.type == token_type_symbol
+                && strcmp (token2.string, ".") == 0)
+              {
+                token_ty token3;
+                x_vala_lex (&token3);
+                if (token3.type == token_type_symbol
+                    && (strcmp (token3.string, "printf") == 0
+                        || strcmp (token3.string, "vprintf") == 0))
+                  {
+                    /* Mark the messages found in the region as c-format
+                       a posteriori.  */
+                    set_format_flag_on_region (inner_region,
+                                               XFORMAT_PRIMARY, yes_according_to_context);
+                  }
+                x_vala_unlex (&token3);
+              }
+            x_vala_unlex (&token2);
+          }
           next_context_iter = null_context_list_iterator;
           state = 0;
           break;
@@ -1293,6 +1459,7 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
           if (delim == token_type_rparen || delim == token_type_eof)
             {
               arglist_parser_done (argparser, arg);
+              unref_region (inner_region);
               return false;
             }
 
@@ -1302,16 +1469,72 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
 
         case token_type_comma:
           arg++;
-          inner_context =
-            inherited_context (outer_context,
-                               flag_context_list_iterator_advance (
-                                 &context_iter));
+          unref_region (inner_region);
+          curr_context = flag_context_list_iterator_advance (&context_iter);
+          inner_region = new_sub_region (outer_region, curr_context);
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
           continue;
 
+        case token_type_question:
+          /* In an expression A ? B : C, each of A, B, C is a distinct
+             sub-region, and since the value of A is not the value of entire
+             expression, if later set_format_flag_on_region is called on this
+             region or an ancestor region, it shall not have an effect on the
+             remembered messages of A.  */
+          inner_region->inherit_from_parent_region = false;
+          unref_region (inner_region);
+          inner_region = new_sub_region (outer_region, curr_context);
+          next_context_iter = passthrough_context_list_iterator;
+          state = 0;
+          continue;
+
+        case token_type_colon:
+          /* In an expression A ? B : C, each of A, B, C is a distinct
+             sub-region.  */
+          unref_region (inner_region);
+          inner_region = new_sub_region (outer_region, curr_context);
+          next_context_iter = passthrough_context_list_iterator;
+          state = 0;
+          continue;
+
+        case token_type_assign:
+          /* In an expression A = B, A and B are distinct sub-regions.
+             The value of B is the value of the entire expression.  */
+          inner_region->inherit_from_parent_region = false;
+          unref_region (inner_region);
+          inner_region = new_sub_region (outer_region, curr_context);
+          next_context_iter = passthrough_context_list_iterator;
+          state = 0;
+          continue;
+
+        case token_type_plus:
+        case token_type_arithmetic_operator:
+        case token_type_equality_test_operator:
+        case token_type_logic_operator:
+        case token_type_compound_assign:
+          /* When an expression contains one of these operators, neither the
+             value on the left of the operator nor the value on the right of the
+             operator is string-valued and the value of the entire expression.
+             Therefore, if later set_format_flag_on_region is called on this
+             region or an ancestor region, it shall not have an effect on the
+             remembered messages of this region.  */
+          inner_region->inherit_from_parent_region = false;
+          unref_region (inner_region);
+          inner_region = new_sub_region (outer_region, curr_context);
+          inner_region->inherit_from_parent_region = false;
+          next_context_iter = passthrough_context_list_iterator;
+          state = 0;
+          continue;
+
+        case token_type_semicolon:
+          arglist_parser_done (argparser, arg);
+          unref_region (inner_region);
+          return false;
+
         case token_type_eof:
           arglist_parser_done (argparser, arg);
+          unref_region (inner_region);
           return true;
 
         case token_type_string_literal:
@@ -1326,7 +1549,7 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
                 char *string = mixed_string_contents (token.mixed_string);
                 mixed_string_free (token.mixed_string);
                 remember_a_message (mlp, NULL, string, true, false,
-                                    inner_context, &pos,
+                                    inner_region, &pos,
                                     NULL, token.comment, false);
               }
             else
@@ -1338,14 +1561,14 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
                     tmp_argparser = arglist_parser_alloc (mlp, next_shapes);
 
                     arglist_parser_remember (tmp_argparser, 1,
-                                             token.mixed_string, inner_context,
+                                             token.mixed_string, inner_region,
                                              pos.file_name, pos.line_number,
                                              token.comment, false);
                     arglist_parser_done (tmp_argparser, 1);
                   }
                 else
                   arglist_parser_remember (argparser, arg,
-                                           token.mixed_string, inner_context,
+                                           token.mixed_string, inner_region,
                                            pos.file_name, pos.line_number,
                                            token.comment, false);
               }
@@ -1355,17 +1578,14 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
           state = 0;
           continue;
 
+        case token_type_return:
+          next_context_iter = passthrough_context_list_iterator;
+          state = 0;
+          continue;
+
         case token_type_character_constant:
         case token_type_lbrace:
         case token_type_rbrace:
-        case token_type_assign:
-        case token_type_return:
-        case token_type_plus:
-        case token_type_arithmetic_operator:
-        case token_type_equality_test_operator:
-        case token_type_logic_operator:
-        case token_type_question:
-        case token_type_colon:
         case token_type_number:
         case token_type_string_template:
         case token_type_regex_literal:
@@ -1401,14 +1621,17 @@ extract_vala (FILE *f,
   phase3_pushback_length = 0;
   last_token_type = token_type_other;
 
+  phase4_pushback_length = 0;
+
   flag_context_list_table = flag_table;
+  nesting_depth = 0;
 
   init_keywords ();
 
   /* Eat tokens until eof is seen.  When extract_parenthesized returns
      due to an unbalanced closing parenthesis, just restart it.  */
   while (!extract_balanced (mlp, token_type_eof,
-                            null_context, null_context_list_iterator,
+                            null_context_region (), null_context_list_iterator,
                             arglist_parser_alloc (mlp, NULL)))
     ;
 
